@@ -1,4 +1,4 @@
-/*	$OpenBSD: uuencode.c,v 1.10 2001/02/08 19:30:53 itojun Exp $	*/
+/*	$OpenBSD: uuencode.c,v 1.12 2001/03/01 02:27:18 deraadt Exp $	*/
 
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
@@ -30,7 +30,7 @@
 
 #include <resolv.h>
 
-RCSID("$OpenBSD: uuencode.c,v 1.10 2001/02/08 19:30:53 itojun Exp $");
+RCSID("$OpenBSD: uuencode.c,v 1.12 2001/03/01 02:27:18 deraadt Exp $");
 
 int
 uuencode(u_char *src, u_int srclength,
